import json
import pathlib

import typer
from rich.console import Console
from rich.theme import Theme

try:
    from wa2rasa import (
        read_wa_object,
        intents_parser,
        entities_parser,
        save_rasa_yaml_file,
    )
except:  # for pytest
    from .wa2rasa import (
        read_wa_object,
        intents_parser,
        entities_parser,
        save_rasa_yaml_file,
    )

app = typer.Typer(
    name="wa2rasa",
    add_completion=False,
    help="Convert Watson Assistant skill object to rasa nlu.yml file.",
    pretty_exceptions_show_locals=False,
)

custom_theme = Theme({"info": "dim cyan", "warning": "magenta", "danger": "bold red"})
console = Console(theme=custom_theme)


@app.command()
def introduce_app(name: str = typer.Argument("bro", help="Your name.")):
    """App intro."""
    console.print(
        f"Hey {name}! This app helps you to convert your watson Assistant skill object to rasa nlu.yml file. For that, use the argument 'convert'.",
        style="info",
    )


@app.command()
def convert(
    wa_file: str = typer.Argument(
        ..., help="The path to your Watson Assistant json file."
    ),
    save_in: str = typer.Argument(
        ..., help="Path to the directory where you want to store the output."
    ),
    rasa_version: str = typer.Argument("3.1", help="The version of your rasa app."),
):
    """
    Convert Watson Assistant skill object to rasa nlu.yml file.
    """
    file_path = pathlib.Path(wa_file)
    if not file_path.absolute().is_file():
        console.print(f"File not found.", style="danger")
        raise Exception(
            f"Please verify that the file {str(file_path.absolute())} exists."
        )
    wa_object = read_wa_object(file_path=file_path.absolute())
    intents = wa_object.get("intents", [])
    entities = wa_object.get("entities", [])
    if intents:
        intents = intents_parser(intents)
    if entities:
        entities = entities_parser(entities)
    nlu_yml_obj = {"version": rasa_version, "nlu": intents + entities}
    if not entities:
        console.print(
            ":warning: No entity found in your Watson Assistant object.",
            style="warning",
        )
    if not intents:
        console.print(
            ":warning: No entities found in your Watson Assistant object.",
            style="warning",
        )
    file_path2 = pathlib.Path(save_in)
    if not file_path2.is_dir():
        console.print(f"Directory not found.", style="danger")
        raise Exception(
            f"Please verify that the directory {str(file_path2.absolute())} exists."
        )
    file_path2 = pathlib.Path(file_path2.absolute(), "nlu.yml")
    save_rasa_yaml_file(file_path=file_path2, object=nlu_yml_obj)
    console.print(f"Output saved in '{file_path2}'.", style="info")


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wa2rasa']

package_data = \
{'': ['*'], 'wa2rasa': ['my_rasa_dir/*', 'wa_example/My-WA-Skill-dialog.json']}

install_requires = \
['PyYAML>=6.0,<7.0', 'rich>=12.5.1,<13.0.0', 'typer>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['wa2rasa = wa2rasa.cli:app']}

setup_kwargs = {
    'name': 'wa2rasa',
    'version': '0.1.1',
    'description': 'Convert Watson Assistant skill object to rasa nlu.yml file.',
    'long_description': 'Purpose of the Package\n======================\n\nConvert intents and entities defined in the Watson Assistant skill\nobject to rasa nlu.yml file.\n\nGiven how rasa works, there is no way to translate dialog flows defined\nin WA into rasa stories. So this converter don’t intends to take care of\ndialog flows.\n\nInstallation\n============\n\nYou can use pip:\n\n.. code:: bash\n\n   $ pip3 install wa2rasa\n\n*Rasa* and *wa2rasa* use common libraries, to avoid conflicts please\ninstall *wa2rasa* in a separate virtual environment.\n\nUsage\n=====\n\nJust run the following command:\n\n.. code:: bash\n\n   $ wa2rasa convert <path_to_your_wa_object>/ <directory_to_store_rasa_nlu_file>/\n\nHere a gif for you:\n\n.. figure:: https://media.giphy.com/media/zQxXPs9HhNJHZBI1Iy/giphy.gif\n   :alt: how to use package\n\n   how to use package\n\nAuthor\n======\n\n`Cloves Paiva <https://www.linkedin.com/in/cloves-paiva-02b449124/>`__.\n',
    'author': 'Cloves Paiva',
    'author_email': 'clovesgtx@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/SClovesgtx/wa2rasa',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json

import yaml

# https://stackoverflow.com/questions/45004464/yaml-dump-adding-unwanted-newlines-in-multiline-strings
class PSS(str):
    pass


def pss_representer(dumper, data):
    style = "|"
    tag = "tag:yaml.org,2002:str"
    return dumper.represent_scalar(tag, data, style=style)


yaml.add_representer(PSS, pss_representer, Dumper=yaml.SafeDumper)


def read_wa_object(file_path):
    with open(file_path, "r") as file:
        wa_object = json.load(file)
    return wa_object


def save_rasa_yaml_file(file_path, object):
    outputfile = open(file_path, "w")
    yaml.safe_dump(object, outputfile, allow_unicode=True, sort_keys=False)
    outputfile.close()


def clean_example(example):
    return example.replace("'", "").replace('"', "").replace(":", "")


def intents_parser(intents):
    for i, intent in enumerate(intents):
        clean_examples = [
            "- " + clean_example(example["text"]) + "\n"
            for example in intent["examples"]
        ]
        intents[i]["examples"] = PSS("".join(clean_examples))
        if type(intent.get("description")) == str:
            del intents[i]["description"]
    return intents


def build_rasa_entity(entity_name, value_name, values, type="synonym"):
    examples = PSS("".join(["- " + value + "\n" for value in values]))
    rasa_entity = {
        type: f"{entity_name}_{value_name}",
        "examples": examples,
    }
    return rasa_entity


def entities_parser(entities):
    obj_entities = []
    for entity in entities:
        entity_name = entity["entity"]
        for value in entity["values"]:
            if value["type"] == "synonyms":
                value_name = value["value"]
                values = value.get("synonyms", []) + [value_name]
                rasa_entity = build_rasa_entity(entity_name, value_name, values)
                obj_entities.append(rasa_entity)
            if value["type"] == "patterns":
                value_name = value["value"]
                values = value["patterns"]
                rasa_entity = build_rasa_entity(
                    entity_name, value_name, values, type="regex"
                )
                obj_entities.append(rasa_entity)
    return obj_entities
